<template>
  <div class="app-container">
    <div style="display: flex;justify-content: space-between">
      <el-form :inline="true" style="display: flex;align-items: center;flex-wrap: wrap;" >
<!--        <el-form-item>-->
<!--          <el-button-->
<!--              type="primary"-->
<!--              plain-->
<!--              icon="Plus"-->
<!--              @click="openDialog('add',{})"-->
<!--          >新增</el-button>-->
<!--        </el-form-item>-->
        <el-form-item v-if="isAdmin" label="企业：" >
          <el-select v-model="data.queryParams.companyId" placeholder="请选择" clearable>
            <el-option
                v-for="item in companyList"
                :key="item.id"
                :label="item.name"
                :value="item.id">
            </el-option>
          </el-select>
        </el-form-item>
        <el-form-item >
          <el-button v-if="isAdmin" type="primary" @click="getList">查询</el-button>
          <el-button v-if="isAdmin" type="primary" plain @click="reset">重置</el-button>
          <el-button type="primary" @click="addFile">生成质量手册</el-button>
        </el-form-item>
      </el-form>
    </div>
    <!-- 表格数据 -->
    <el-table v-loading="loading" :data="dataList" :border="true">
      <el-table-column label="序号" type="index" align="center" width="80"/>
      <el-table-column label="质量手册" prop="fileName" align="center">
        <template #default="scope">
          <el-button link type="primary" @click="initFile(scope.row.companyName)">{{scope.row.companyName + scope.row.qualityName + '.docx'}}</el-button>
        </template>
      </el-table-column>
      <el-table-column label="操作" align="center" class-name="small-padding fixed-width">
        <template #default="scope">
<!--          <el-button link type="primary" @click="openDialog('edit',scope.row)">编辑</el-button>-->
          <el-button link type="danger" @click="handleDelete(scope.row)">删除</el-button>
        </template>
      </el-table-column>
    </el-table>

    <div class="orgTreeBox" id="org-tree-container">
      <vue3-tree-org
          :data="deptList"
          :horizontal="false"
          :props="treeProps"
          :toolBar="tools"
          :label-style="labelStyle"
          center
      />
    </div>
    <pagination
        v-show="total > 0"
        :total="total"
        v-model:page="queryParams.pageNum"
        v-model:limit="queryParams.pageSize"
        @pagination="getList"
    />
  </div>
</template>

<script setup>
import {getCurrentInstance, onMounted, onUnmounted, reactive, ref, toRefs} from "vue";
import {ElMessage, ElMessageBox} from "element-plus";
import {delCompany, getCompany} from "@/api/onlineEducation/company";
import {generateWordDocument} from './components/exportDoc.js'
// import OrgTree from 'vue-org-tree'
import html2canvas from 'html2canvas'
import { saveAs } from 'file-saver'
import useUserStore from "@/store/modules/user";
import {
  addStandardQuality,
  delStandardQuality,
  getStandardDetail,
  getStandardQuality
} from "@/api/standardSys/standardSys";
import {getDepart, getDistribution} from "@/api/orgStructure/depart";

const userStore = useUserStore()
const { proxy } = getCurrentInstance();
const loading = ref(false);
const dialogRef = ref();
const data = reactive({
  queryParams: {
    pageNum: 1,
    pageSize: 10,
    companyId: null
  },
  total: 0,
  dataList: [],
  deptList: {
    id: 0,
    deptName: "总经理",
    children:[
      {
        id: '0-1',
        deptName: "管理者代表",
        children: []
      }
    ]
  },
  treeProps: {
    label: 'deptName'
  },
  tools: {
    scale: false, restore: false, expand: false, zoom: false, fullscreen: false
  },
  labelStyle: {
    border: '1px solid #ccc',
    background: 'rgba(0,0,0,0)'
  },
  companyList: [],
  isAdmin: false,
  companyInfo: {
    summaries: '',
    policies: '',
    deptList: [],
    duties: [],
    temps: []
  },
  caluseList: [
    {clauseNum: '4', content: '组织环境'},
    {clauseNum: '4.1', content: '理解组织及其环境',manage: true,represent: false},
    {clauseNum: '4.2', content: '理解相关方的需求和期望',manage: true,represent: false},
    {clauseNum: '4.3', content: '确定质量管理体系的范围',manage: false,represent: true},
    {clauseNum: '4.4', content: '质量管理体系及其过程',manage: false,represent: true},
    {clauseNum: '5', content: '领导作用'},
    {clauseNum: '5.1', content: '领导作用和承诺',manage: true,represent: false},
    {clauseNum: '5.2', content: '质量方针',manage: true,represent: false},
    {clauseNum: '5.3', content: '组织的岗位、职责和权限',manage: true,represent: false},
    {clauseNum: '6', content: ''},
    {clauseNum: '6.1', content: '应对风险和机遇的措施',manage: true,represent: false},
    {clauseNum: '6.2', content: '质量目标及其实现的策划',manage: false,represent: true},
    {clauseNum: '6.3', content: '变更的策划',manage: false,represent: true},
    {clauseNum: '7', content: '支持'},
    {clauseNum: '7.1', content: '资源',manage: true,represent: false},
    {clauseNum: '7.1.1', content: '总则',manage: true,represent: false},
    {clauseNum: '7.1.2', content: '人员',manage: true,represent: false},
    {clauseNum: '7.1.3', content: '基础设施',manage: false,represent: true},
    {clauseNum: '7.1.4', content: '过程运行环境',manage: false,represent: true},
    {clauseNum: '7.1.5', content: '监视和测量资源',manage: false,represent: true},
    {clauseNum: '7.1.6', content: '组织的知识',manage: false,represent: true},
    {clauseNum: '7.2', content: '能力'},
    {clauseNum: '7.3', content: '意识'},
    {clauseNum: '7.4', content: '沟通'},
    {clauseNum: '7.5', content: '成文信息'},
    {clauseNum: '7.6', content: '质量信息'},
    {clauseNum: '8', content: '运行'},
    {clauseNum: '8.1', content: '运行策划和控制'},
    {clauseNum: '8.2', content: '产品和服务的要求'},
    {clauseNum: '8.2.1', content: '顾客沟通'},
    {clauseNum: '8.2.2', content: '与产品和服务有关的要求的确定'},
    {clauseNum: '8.2.3', content: '与产品和服务有关的要求的评审'},
    {clauseNum: '8.2.4', content: '产品和服务要求的更改'},
    {clauseNum: '8.3', content: '产品和服务的设计和开发'},
    {clauseNum: '8.3.1', content: '总则'},
    {clauseNum: '8.3.2', content: '设计和开发策划'},
    {clauseNum: '8.3.3', content: '设计和开发输入'},
    {clauseNum: '8.3.4', content: '设计和开发控制'},
    {clauseNum: '8.3.5', content: '设计和开发输出'},
    {clauseNum: '8.3.6', content: '设计和开发更改'},
    {clauseNum: '8.3.7', content: '新产品试制'},
    {clauseNum: '8.3.8', content: '设计和开发的试验控制'},
    {clauseNum: '8.4', content: '外部提供过程、产品和服务的控制'},
    {clauseNum: '8.4.1', content: '总则'},
    {clauseNum: '8.4.2', content: '控制类型和程度'},
    {clauseNum: '8.4.3', content: '提供给外部供方的信息'},
    {clauseNum: '8.5', content: '生产和服务提供'},
    {clauseNum: '8.5.1', content: '生产和服务提供的控制'},
    {clauseNum: '8.5.2', content: '标识和可追溯性'},
    {clauseNum: '8.5.3', content: '顾客或外部供方的财产'},
    {clauseNum: '8.5.4', content: '防护'},
    {clauseNum: '8.5.5', content: '交付后的活动'},
    {clauseNum: '8.5.6', content: '更改控制'},
    {clauseNum: '8.5.7', content: '关键过程'},
    {clauseNum: '8.6', content: '产品和服务的放行'},
    {clauseNum: '8.7', content: '不合格输出的控制'},
    {clauseNum: '9', content: '绩效评价'},
    {clauseNum: '9.1', content: '监视、测量、分析和评价'},
    {clauseNum: '9.1.1', content: '总则'},
    {clauseNum: '9.1.2', content: '顾客满意'},
    {clauseNum: '9.1.3', content: '分析和评价'},
    {clauseNum: '9.2', content: '内部审核'},
    {clauseNum: '9.3', content: '管理评审'},
    {clauseNum: '10', content: '持续改进'},
    {clauseNum: '10.1', content: '总则'},
    {clauseNum: '10.2', content: '不合格和纠正措施'},
    {clauseNum: '10.3', content: '持续改进'}
  ]
});

const { queryParams, total, dataList,deptList,treeProps,tools,labelStyle,companyList, isAdmin, companyInfo, caluseList } = toRefs(data);
const userInfo = ref()
onMounted(async ()=>{
  if(userStore.roles.includes('admin')){
    data.isAdmin = true
    await getCompanyList()
  }else{
    data.isAdmin = false
    data.queryParams.companyId = userStore.companyId
  }
  await getList()
})

onUnmounted(()=>{

})


const getList = async () => {
  loading.value = true
  const res = await getStandardQuality(data.queryParams)
  if(res.code == 200){
    data.dataList = res.data.data || []
    data.total = res.data.total
  }else{
    ElMessage.warning(res.message)
  }
  loading.value = false
}

const getDeptData = async (val) => {
  await getDeptList(val.companyId)
  const res = await getDistribution({companyId: val.companyId})
  if(res.code == 200){
    for(let item of data.caluseList){
      const sameNum = data.companyInfo.duties.filter(i=>i.clauseNum == item.clauseNum).map(j=> {
            return {
              deptId: j.deptId,
              chooseLab: j.chooseLab
            }
          }
      )
      for(let i of sameNum){
        item[i.deptId] = i.chooseLab? (i.chooseLab==1?'●':'○'):'○'
      }
    }
  }else{
    ElMessage.warning(res.message)
  }
}
const getDeptList = async (id) => {
  const res = await getDepart({pageNum: 1, pageSize: 999, companyId: id})
  if(res.code == 200){
    data.deptList.children[0].children = res.data
  }else{
    ElMessage.warning(res.message)
  }
}
const addFile = async ()=>{
  let params={}
  if(data.queryParams.companyId){
    params = {
      qualityName: '质量手册',
      companyId: data.queryParams.companyId
    }
  }else{
    ElMessage.warning('请先选择对应的企业')
    return
  }
  const res = await addStandardQuality(params)
  if(res.code == 200){
    ElMessage.success(res.message)
    await getList()
  }else{
    ElMessage.warning(res.message)
  }
}

const getCompanyList = async ()=>{
  const queryParams = {
    pageNum: 1,
    pageSize: 999
  }
  const res = await getCompany(queryParams)
  if (res.code == 200) {
    data.companyList = res.data.list?res.data.list:[]
    // data.queryParams.companyId = data.companyList[0].id
  } else {
    ElMessage.warning(res.message)
  }
}

const getInfo = async (val) => {
  loading.value = true;
  const res = await getStandardDetail({companyId: val.companyId});
  if(res.code === 200){
    if(!res.data || (res.data.companyIndustryTemplates.length == 0 && res.data.companyQualityPolicies.length == 0 && res.data.companySummaries.length == 0 && res.data.sysFunctionalDistributions
        .length == 0 && res.data.treeSelects.length == 0)){
      loading.value = false;
      return Promise.reject(new Error('该企业暂无质量数据'));
    }
    data.companyInfo.summaries = res.data.companySummaries ? res.data.companySummaries[0]?.companySummary : []
    data.companyInfo.policies = res.data.companyQualityPolicies ? res.data.companyQualityPolicies[0]?.policy : []
    data.companyInfo.deptList = res.data.treeSelects ? res.data.treeSelects : []
    // data.companyInfo.clauses = data.caluseList
    // data.companyInfo.duties = res.data.sysFunctionalDistributions

    const duties = transToTableData(res.data.sysFunctionalDistributions)
    data.companyInfo.allDeptNames = duties.allDeptNames
    data.companyInfo.clauses = duties.clauses
    data.companyInfo.temps = res.data.companyIndustryTemplates?.map((item,index)=>{
      return {
        index: index + 1,
        templateName: item.templateName
      }
    }) || []
    // const imageBase64 = await exportTableToImage()
    // data.companyInfo.tableImage = {
    //   data: imageBase64.split(",")[1], // 去掉 data:image/png;base64, 前缀
    //   type: "png",
    // }
  }else{
    ElMessage.warning(res.message)
  }
  loading.value = false;
}

const transToTableData=(duties)=>{
  // 步骤1：获取所有唯一的部门和条款编号
  const allDeptNames = [...new Set(duties.map(item => item.deptName))];
  const allClauseNums = [...new Set([
    ...data.caluseList.map(c => c.clauseNum),
    ...duties.map(d => d.clauseNum)
  ])];

// 步骤2：为每个条款生成完整的部门数据（缺失数据默认 chooseLab: 0）
  const processedClauses = allClauseNums.map(clauseNum => {
    const clauseContent = data.caluseList.find(c => c.clauseNum === clauseNum)?.content || "";

    // 为当前条款生成所有部门的数据（确保每个部门都有值）
    const deptValues = allDeptNames.map(deptName => {
      const matchedDept = duties.find(
          item => item.clauseNum === clauseNum && item.deptName === deptName
      );
      return matchedDept ? (matchedDept.chooseLab==1?'●':'○' ): '○';

    });

    return {
      clauseNum,
      content: clauseContent,
      deptValues // 数组形式，例如 [0, 1, 0, 0]
    };
  });

// 最终数据结构
  return {
    clauses: processedClauses,
    allDeptNames // 用于生成表头
  };
}

const initFile = async (val) => {
  try {
    await getInfo(val)
    await getDeptData(val)
    // 2. 等待DOM更新完成
    await nextTick();

    // 3. 捕获组织结构图图片
    const orgChartImage = await captureElementToImage('org-tree-container');
    data.companyInfo.orgChart = orgChartImage
    const templatePath = '/qualityFile.docx'
    await generateWordDocument(
        templatePath,
        data.companyInfo,
        val.companyName+'质量手册.docx'
    );
    ElMessage.success('手册导出成功！')
  } catch (error) {
    console.error('操作失败:', error);
    ElMessage.warning(error.message || '操作失败');
  }
}

const captureElementToImage = async (elementId) => {
  const html2canvas = (await import('html2canvas')).default;
  const element = document.getElementById(elementId);
  if (!element) throw new Error('找不到组织结构图容器');

  return await html2canvas(element, {
    scale: 4,
    useCORS: true,
    allowTaint: true,
    backgroundColor: 'rgba(0,0,0,0)' // 确保背景是白色
  }).then(canvas => canvas.toDataURL('image/png'));
}

const openDialog = (type, value) => {
  dialogRef.value.openDialog(type, value, data.queryParams.companyId, data.isAdmin, data.companyList);
}

/** 重置新增的表单以及其他数据  */
const reset= async()=> {
  data.queryParams = {
    pageNum: 1,
    pageSize: 10,
    companyId: null
  }
  await getCompanyList()
  await getList()
}
const handleDelete = (val) => {
  ElMessageBox.confirm(
      '确定删除此条数据？',
      '提示',
      {
        confirmButtonText: '确定',
        cancelButtonText: '取消',
        type: 'warning',
      })
      .then( async() => {
        const res = await delStandardQuality({standardizedQualityId: val.id})
        if(res.code == 200){
          ElMessage.success('数据删除成功')
          await getList()
        }else{
          ElMessage.warning(res.message)
        }
      })
}

</script>
<style lang="scss">
  .orgTreeBox{
    width: 700px;
    height: 700px;
    position: absolute;  /* 或 fixed */
    left: -9999px;      /* 移出可视区域 */
    pointer-events: none; /* 禁止交互 */
    z-index: -1;
  }
</style>